// ==============================================================
//                 ORBITER MODULE: LagrangeLaunchMFD
//                  Part of the ORBITER SDK
//           Copyright (C) 2002-2003 Martin Schweiger
//                   All rights reserved
//
// LagrangeLaunchMFD.cpp
// Reference implementation of generic user-defined MFD mode
// ==============================================================

#define STRICT
#define ORBITER_MODULE
#include <windows.h>
#include <stdio.h>
#include <cmath>
#include "orbitersdk.h"
#include "LagrangeLaunchMFD.h"

// ==============================================================
// Global variables

int status = 0;
int lp_tgt = 1;
double req_azimuth = PI05;
double req_inc = 0;
double launch_time = 0;
double burn_time = 0;
double req_dV = 0;
double burn_duration = 0;
bool no_intersect = false;
double azimuth = 0;

double steptime = 0;
double post_step = 0;
int node_mode = 0;
int warning_sw = 0;
double angle_to_pe = 0;
double time_to_pe = 0;
bool has_main_eng = true;
double parking_alt = 200;

double mu = 398600439968871.2;
double rt = 6371010;
VECTOR3 GBOD_AXIS = _V(0,0,0);
double earth_rot_rate = PI2/86164;

VESSEL2* vme;
ELEMENTS el;
ORBITPARAM prm;

char vesname[256];
char gbodyname[256];

OBJHANDLE MyVes;
OBJHANDLE gBod;




static struct {  // "Ascent MFD" parameters
	int mode;      // identifier for new MFD mode
	
} g_LagrangeLaunchMFD;

LagrangeLaunchMFD::SavePrm LagrangeLaunchMFD::saveprm = {0,0,0,0};

// ==============================================================
// API interface



DLLCLBK void InitModule (HINSTANCE hDLL)
{
	
	static char *name = "LagrangeLaunchMFD";
	MFDMODESPEC spec;
	spec.name    = name;
	spec.key     = OAPI_KEY_P;
//	spec.context = NULL;
	spec.msgproc = LagrangeLaunchMFD::MsgProc;
	g_LagrangeLaunchMFD.mode = oapiRegisterMFDMode (spec);
	/////////////////////////////////////
	//sprintf(oapiDebugString(), "%d", spec.msgproc);

	
}


DLLCLBK void ExitModule (HINSTANCE hDLL)
{
	oapiUnregisterMFDMode (g_LagrangeLaunchMFD.mode);
}

DLLCLBK void opcCloseRenderViewport() // called before StoreStatus()
{

	LagrangeLaunchMFD::saveprm.sp1 = 0;
	LagrangeLaunchMFD::saveprm.sp2 = 0;
	LagrangeLaunchMFD::saveprm.sp3 = 0;
	LagrangeLaunchMFD::saveprm.sp4 = 0;

	strcpy(vesname,"NULL");
	strcpy(gbodyname,"NULL");
	MyVes = NULL;
	gBod = NULL;

	status = 0;
    lp_tgt = 1;
	req_azimuth = PI05;
	req_inc = 0;
	launch_time = 0;
	node_mode = 0;
	burn_time = 0;
	req_dV = 0;
	burn_duration = 0;
	angle_to_pe = 0;
	time_to_pe = 0;
	parking_alt = 200;
	
}


DLLCLBK void opcPostStep (double simt, double simdt, double mjd)
{
post_step = simdt;

}

DLLCLBK void opcPreStep (double SimT, double SimDT, double mjd)
{
steptime = SimDT;
	
///////////// init gbody ////////////////////
if(MyVes)
	{

	if (strlen(gbodyname) == 0)
		{
		gBod = vme->GetGravityRef();
		oapiGetObjectName(gBod, gbodyname, 30);
		LagrangeLaunchMFD::GetGbodData();
		}
	else if (!strnicmp (gbodyname, "NULL", 4))
		{
		gBod = vme->GetGravityRef();
		oapiGetObjectName(gBod, gbodyname, 30);
		LagrangeLaunchMFD::GetGbodData();
		}
	else// if(!gBod)
		{
		gBod = oapiGetObjectByName(gbodyname);
		if(!gBod)
			{
			gBod = vme->GetGravityRef();
			oapiGetObjectName(gBod, gbodyname, 30);
			}
	LagrangeLaunchMFD::GetGbodData();
		}
/////////////////////////do stuff ///////////////////////////////////////////////////

	if(status == 1)
	{
		VECTOR3 earth_pos_glob;
		oapiGetGlobalPos(gBod, &earth_pos_glob);
		VECTOR3 earth_pos_glob_nm = earth_pos_glob;
		normalise(earth_pos_glob_nm);

		//if(lp_tgt == 1)earth_pos_glob_nm = -earth_pos_glob_nm;

		VECTOR3 ECLIP_AXIS = _V(0,1,0);
		double lan_angle = -(76*RAD);//76
		if(lp_tgt == 1) lan_angle = (104*RAD);//104
		MATRIX3 rm = rotm(ECLIP_AXIS, lan_angle);

		VECTOR3 orbit_lan_glob = mul(rm, earth_pos_glob_nm);
		normalise(orbit_lan_glob);



		// lissajous_radius = 750000;// km
		//double liss_angle = asin(lissajous_radius/1500000);
		double liss_angle = (24*RAD);/// 24
		//if(lp_tgt == 1)liss_angle = (24*RAD);//24

		rm = rotm(orbit_lan_glob, -liss_angle);
		VECTOR3 orbit_axis_glob = mul(rm, ECLIP_AXIS);
		normalise(orbit_axis_glob);

		normalise(GBOD_AXIS);

		VECTOR3 orbit_lan_equ = crossp(orbit_axis_glob,GBOD_AXIS);
		normalise(orbit_lan_equ);

		req_inc = acos(dotp(GBOD_AXIS, orbit_axis_glob));

		double lon,lat,rad;
		vme->GetEquPos(lon,lat,rad);

		if(abs(req_inc) < abs(lat))no_intersect = true;
		else no_intersect = false;

		double sin_beta = cos(req_inc)/cos(lat);

		if(sin_beta > 1) sin_beta = 1;
		else if(sin_beta < -1) sin_beta = -1;

		double beta = asin(sin_beta);

		double delta_lat_angle1 = atan(sin(lat)*tan(beta));
		double delta_lat_angle2 = PI - delta_lat_angle1;


		VECTOR3 ves_pos_rel;
		vme->GetRelativePos(gBod,ves_pos_rel);
		normalise(ves_pos_rel);

		VECTOR3 ves_pos_norm = crossp(GBOD_AXIS, ves_pos_rel);
		normalise(ves_pos_norm);

		VECTOR3 ves_pos_mapped = crossp(ves_pos_norm, GBOD_AXIS);
		normalise(ves_pos_mapped);

		double lt_angle = 200*earth_rot_rate;
		rm = rotm(GBOD_AXIS, -lt_angle);
		ves_pos_mapped = mul(rm,ves_pos_mapped);
		normalise(ves_pos_mapped);

		double vpm_orblanequ_angle = acos(dotp(ves_pos_mapped, orbit_lan_equ));

		VECTOR3 vpm_orblanequ_nm = crossp(ves_pos_mapped, orbit_lan_equ);
		normalise(vpm_orblanequ_nm);
		double pole_vpmon_angle = acos(dotp(GBOD_AXIS, vpm_orblanequ_nm));
		if(pole_vpmon_angle > PI05)vpm_orblanequ_angle = PI2 - vpm_orblanequ_angle;

		

		double tot_angle1 = PI2 - vpm_orblanequ_angle + delta_lat_angle1;
		double tot_angle2 = PI2 - vpm_orblanequ_angle + delta_lat_angle2;
		
		if(tot_angle1 > PI2)tot_angle1 = tot_angle1 - PI2;
		if(tot_angle2 > PI2)tot_angle2 = tot_angle2 - PI2;

	//	sprintf(oapiDebugString(), "%0.2f %0.2f %0.2f", vpm_orblanequ_angle*DEG, delta_lat_angle2*DEG, tot_angle2*DEG);

		if(node_mode == 1)launch_time = tot_angle1/earth_rot_rate;
		else launch_time = tot_angle2/earth_rot_rate;
		launch_time = -launch_time;

	/// azimuth
	VECTOR3 relvel;
	vme->GetRelativeVel(gBod,relvel);
	VECTOR3 relvel_glob = relvel+earth_pos_glob;
	VECTOR3 rel_vel_loc = _V(0,0,0);
	vme->Global2Local(relvel_glob,rel_vel_loc);

	VECTOR3 ov_hrz = _V(0,0,0);
	vme->HorizonRot(rel_vel_loc, ov_hrz);

	double orb_ground_spd = sqrt((ov_hrz.x*ov_hrz.x)+(ov_hrz.z*ov_hrz.z));

	VECTOR3 orb_ground_vector = _V(ov_hrz.x,0,ov_hrz.z);

	double final_azimuth = asin(cos(req_inc)/cos(lat));
	final_azimuth+=(90*RAD-final_azimuth)*(1-(req_inc/abs(req_inc)));

	double po_alt = parking_alt*1000;
	double ap_tgt = 1200000000;
	double finalv = sqrt(mu*(2/(po_alt+rt)-1/(0.5*(ap_tgt+po_alt+(2*rt)))));

	VECTOR3 final_ground_vector = _V((finalv*sin(final_azimuth)),0,(finalv*cos(final_azimuth)));

	VECTOR3 req_ground_vector = final_ground_vector - orb_ground_vector;

	azimuth = atan2(req_ground_vector.x,req_ground_vector.z);

	if(node_mode == 0)azimuth = PI - azimuth;
		

		int fs = vme->GetFlightStatus();
		if(fs == 0)
		{
			status = 2;
			launch_time = 0;
		}

		burn_duration = 0;
		burn_time = 0;
		req_dV = 0;
	}
	else if(status == 2)
	{
		//
		launch_time = 0;

		vme->GetElements (gBod,el,&prm,0,FRAME_ECL);
		double AgP = el.omegab - el.theta;
		double anglepastlan = prm.TrA + AgP;
		if(anglepastlan > PI2)anglepastlan = anglepastlan - PI2;
		else if(anglepastlan < 0)anglepastlan = anglepastlan + PI2;

		/////////////////////////////////////////////////////

		double ves_mass = vme->GetMass();
		int tc = vme->GetGroupThrusterCount(THGROUP_MAIN);
		double main_th = 0;
		double main_isp = 0;
		double fuel_rate = 0;
		burn_duration = 0;
		req_dV = 0;
		burn_time = 0;

		if(tc > 0)
		{
		for (int i = 0; i < tc; i++)
			{
			THRUSTER_HANDLE hThruster = vme->GetGroupThruster(THGROUP_MAIN, i);
			double th = vme->GetThrusterMax(hThruster);
			main_th = main_th + th;
			main_isp = vme->GetThrusterIsp(hThruster);
			}

		fuel_rate = main_th/main_isp;
		VECTOR3 relpos;
		vme->GetRelativePos(gBod, relpos);
		double alt = length(relpos);
		VECTOR3 relvel;
		vme->GetRelativeVel(gBod,relvel);
		double vel = length(relvel);
		double ap_tgt = 1200000000;

		double finalv = sqrt(mu*(2/(alt)-1/(0.5*(ap_tgt+alt+(2*rt)))));
		req_dV = finalv - vel;

		double fraction = exp((0.5*req_dV)/main_isp);
		double fuelmass = ves_mass - (ves_mass/fraction);
		burn_duration = fuelmass/fuel_rate;

		double angvel = PI2/prm.T;

		double req_pe = (110*RAD);
		angle_to_pe = req_pe - anglepastlan;
		if(angle_to_pe < 0)angle_to_pe = angle_to_pe + PI2;

		burn_time = -((angle_to_pe/angvel) - burn_duration);
		time_to_pe = (angle_to_pe/angvel);

		fraction = exp(req_dV/main_isp);
		fuelmass = ves_mass - (ves_mass/fraction);
		burn_duration = fuelmass/fuel_rate;

		}
		else has_main_eng = false;


	}
//////////////////////////////////////////////////////////////////////
	}
else 
	{
	LagrangeLaunchMFD::saveprm.sp1 = 0;
	LagrangeLaunchMFD::saveprm.sp2 = 0;
	LagrangeLaunchMFD::saveprm.sp3 = 0;
	LagrangeLaunchMFD::saveprm.sp4 = 0;


	strcpy(vesname,"NULL");
	strcpy(gbodyname,"NULL");
	MyVes = NULL;
	gBod = NULL;

	status = 0;
    lp_tgt = 1;
	req_azimuth = PI05;
	req_inc = 0;
	launch_time = 0;
	node_mode = 0;
	burn_time = 0;
	req_dV = 0;
	burn_duration = 0;
	parking_alt = 200;
	}
}



// ==============================================================
// Ascent MFD implementation

LagrangeLaunchMFD::LagrangeLaunchMFD (DWORD w, DWORD h, VESSEL *vessel)
: GraphMFD (w, h, vessel)
{
//status = 0;
//lp_tgt = 1;
//node_mode = 0;

//sprintf(oapiDebugString(), "Implmentation");

///////// init vessel /////////////////////
	if (strlen(vesname) == 0)
	{
	MyVes = oapiGetFocusObject();
	oapiGetObjectName(MyVes, vesname, 30);
	}
	else
	{
	MyVes = oapiGetVesselByName(vesname);
	}
	if(MyVes)vme = (VESSEL2*)oapiGetVesselInterface(MyVes);
	else
	{
	MyVes = oapiGetFocusObject();
	oapiGetObjectName(MyVes, vesname, 30);
	if(MyVes)vme = (VESSEL2*)oapiGetVesselInterface(MyVes);
	}

	
LagrangeLaunchMFD::StoreStatus ();
}

LagrangeLaunchMFD::~LagrangeLaunchMFD ()
{
//status = 0;
//lp_tgt = 1;
//node_mode = 0;
}



// message parser
int LagrangeLaunchMFD::MsgProc (UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam)
{
	switch (msg) {
	case OAPI_MSG_MFD_OPENED:
		return (int)(new LagrangeLaunchMFD (LOWORD(wparam), HIWORD(wparam), (VESSEL*)lparam));
		//sprintf(oapiDebugString(), "mfd", mfd);
	}
	return 0;
}


bool LagrangeLaunchMFD::ConsumeKeyBuffered (DWORD key)
{
	
	bool cbSelectVessel(void *id, char *str, void *data);
	bool cbSelectPO(void *id, char *str, void *data);

	switch (key) {
	case OAPI_KEY_K:
		oapiOpenInputBox("Enter Parking Orbit Alt.(km)", cbSelectPO, 0, 30, (void *)this);
		return true;
	//case OAPI_KEY_E:
	//	oapiOpenInputBox("Enter Landing Target", cbSelectTarget, 0, 30, (void *)this);
	//	return true;
	case OAPI_KEY_V:
		oapiOpenInputBox("Enter Vessel Name", cbSelectVessel, 0, 30, (void *)this);
		return true;
	case OAPI_KEY_B:
		if(status == 0)status = 1;
		else
		{
			no_intersect = false;
			status = 0;
			launch_time = 0;
			req_inc = 0;
			burn_time = 0;
			burn_duration = 0;
			req_dV = 0;
		}
		InvalidateDisplay();
		return true;
	case OAPI_KEY_N:
		if(lp_tgt == 1)lp_tgt = 2;
		else lp_tgt = 1;
		InvalidateDisplay();
		return true;
	case OAPI_KEY_M:
		if(node_mode == 0)node_mode = 1;
		else node_mode = 0;
		InvalidateDisplay();
		return true;
	}
	return false;
}

bool LagrangeLaunchMFD::ConsumeButton (int bt, int event)
{
	if (!(event & PANEL_MOUSE_LBDOWN)) return false;
	//if(display_sw == 0)
	{
	static const DWORD btkey[12] = { OAPI_KEY_V, OAPI_KEY_B, OAPI_KEY_N, OAPI_KEY_M, OAPI_KEY_K, NULL,
									NULL,NULL,NULL,NULL,NULL,NULL};
	if (bt < 12) return ConsumeKeyBuffered (btkey[bt]);
	else return false;
	}
//	else if(display_sw == 1)
//	{
//	static const DWORD btkey[12] = { OAPI_KEY_K,OAPI_KEY_G, OAPI_KEY_B, OAPI_KEY_J, OAPI_KEY_M, OAPI_KEY_D,
//									OAPI_KEY_V,OAPI_KEY_8,OAPI_KEY_1,OAPI_KEY_2,OAPI_KEY_3,OAPI_KEY_C};
//	if (bt < 12) return ConsumeKeyBuffered (btkey[bt]);
//	else return false;
//	}
	return false;
}

char *LagrangeLaunchMFD::ButtonLabel (int bt)
{
	
	char *label[12] = {"VES", "STS", "LP", "NOD", "PO", "",
						"","","","","",""};
//	char *labelM[12] = {"NOH", "SID", "NOC", "PLD", "TNK", "RCS",
//						"DOK","CAB","NAV","LMP","LMP","DSP"};
//	if(display_sw == 0)return (bt < 12 ? label[bt] : 0);
//	else return (bt < 12 ? labelM[bt] : 0);

	return (bt < 12 ? label[bt] : 0);
}

int LagrangeLaunchMFD::ButtonMenu (const MFDBUTTONMENU **menu) const
{
	//if(display_sw == 0)
	{
	static const MFDBUTTONMENU mnu[12] = {
		{"Select Vessel", 0, 'V'},
		{"On/Off", 0, 'B'},
		{"Select LP", 0, 'N'},
		{"Select Node", 0, 'M'},
		{"Set PO Alt", 0, 'K'},
		{"", 0, ' '},
		{"", 0, ' '},
		{"", 0, ' '},
		{"", 0, ' '},
		{"", 0, ' '},
		{"", 0, ' '},
		{"", 0, ' '}
	};
	if (menu) *menu = mnu;
	return 12;
	}
	//else
	//{
	//static const MFDBUTTONMENU mnu[12] = {
	//	{"Nose Hatch", 0, 'K'},
	//	{"Side Hatch", 0, 'G'},
	//	{"Nose Cone", 0, 'B'},
	//	{"Jettison Payload", 0, 'J'},
	//	{"Jettison Trunk", 0, 'M'},
	//	{"RCS Mode", 0, 'D'},
	//	{"Extend/Retract Dock", 0, 'V'},
	//	{"Cabin Lights", 0, '8'},
	//	{"NAV Beacons", 0, '1'},
	//	{"Fwd.Lamp 1", 0, '2'},
	//	{"Fwd.lamp 2", 0, '3'},
	//	{"Display", 0, 'C'}
	//};
	//if (menu) *menu = mnu;
	//return 12;
	//}
}


bool cbSelectVessel(void *id, char *str, void *data)
{
	return (((LagrangeLaunchMFD *)data)->SelectVessel(str));
}

bool cbSelectPO(void *id, char *str, void *data)
{
	return (((LagrangeLaunchMFD *)data)->SelectPO(str));
}

bool LagrangeLaunchMFD::SelectVessel(char *Name)
{
	MyVes = oapiGetVesselByName(Name);

	if (MyVes)
	{	
	strcpy(vesname,Name);
	vme = (VESSEL2*)oapiGetVesselInterface(MyVes);
	InvalidateDisplay();
	return true;
	}

	return false;
}

bool LagrangeLaunchMFD::SelectPO(char *PO_Alt)
{
	double new_po = 0;
	sscanf_s(PO_Alt, "%lf", &new_po);

	parking_alt = new_po;

	InvalidateDisplay();
	return true;
}


void LagrangeLaunchMFD::GetGbodData()
{
	
	double ob = oapiGetPlanetObliquity (gBod);
	GBOD_AXIS.x = 0;
	GBOD_AXIS.y = cos(ob);
	GBOD_AXIS.z = sin(ob);
	double eqlan = oapiGetPlanetTheta(gBod);
	MATRIX3 rm = rotm(_V(0,1,0),-eqlan);
	GBOD_AXIS = mul(rm,GBOD_AXIS);

	double prd = oapiGetPlanetPeriod(gBod);
	earth_rot_rate = PI2/prd;

	double mass = oapiGetMass(gBod);
	mu = GGRAV*mass;

	rt = oapiGetSize(gBod);
	//sprintf(oapiDebugString(), "ob %0.6f eqlan %0.6f", obl*DEG, eqlanan*DEG);
	
}


void LagrangeLaunchMFD::Update (HDC hDC)
{
	
	SelectDefaultFont(hDC,0);	
	Title (hDC, "LagrangeLaunchMFD");

		
		SelectDefaultFont(hDC,0);
		SetTextColor(hDC, 0x0033FF33);	// green

		TextOut(hDC,(0.5*cw),(ch*2.7),"Vessel", 6);
		{
		SetTextColor(hDC, 0x0033FFFF);	// Yellow
		char kbuf[256];
		sprintf(kbuf,"%s", (vesname) );
		TextOut(hDC,(cw*0.5),(ch*3.7),kbuf,strlen(kbuf));
		}


		SetTextColor(hDC, 0x0033FF33);	// green
		TextOut(hDC,(0.5*cw),(ch*6),"On/Off", 6);
		if(status == 0)
		{
			SetTextColor(hDC, 0x0033FF33);
			TextOut(hDC,(0.5*cw),(ch*7),"OFF", 3);
		}
		else
		{
			SetTextColor(hDC, 0x0033FFFF);
			TextOut(hDC,(0.5*cw),(ch*7),"ACTIVE", 6);
		}
		

		SetTextColor(hDC, 0x0033FF33);	// green
		TextOut(hDC,(0.5*cw),(ch*9.3),"LP Selection", 12);

		if(lp_tgt == 1)
		{
			SetTextColor(hDC, 0x0033FFFF);	// Yellow
			TextOut(hDC,(0.5*cw),(ch*10.3),"SUN-EARTH L1", 12);
		}
		else
		{
			SetTextColor(hDC, 0x0033FFFF);	// Yellow
			TextOut(hDC,(0.5*cw),(ch*10.3),"SUN-EARTH L2", 12);
		}
		
		SetTextColor(hDC, 0x0033FF33);	// green
		TextOut(hDC,(0.5*cw),(ch*12.6),"Select Ascending/Descending Node", 32);
		if(node_mode == 1)
		{
			SetTextColor(hDC, 0x0033FFFF);	// Yellow
			TextOut(hDC,(0.5*cw),(ch*13.6),"Launch on Ascending Node", 24);
		}
		else
		{
			SetTextColor(hDC, 0x0033FFFF);	// Yellow
			TextOut(hDC,(0.5*cw),(ch*13.6),"Launch on Descending Node", 25);
		}


		SetTextColor(hDC, 0x0033FF33);	// green
		TextOut(hDC,(0.5*cw),(ch*15.9),"Set Parking Orbit Alt.", 22);

		{
		SetTextColor(hDC, 0x0033FFFF);	// Yellow
		char kbuf[256];
		sprintf(kbuf,"%0.1f km", parking_alt );
		TextOut(hDC,(cw*0.5),(ch*16.9),kbuf,strlen(kbuf));
		}


		///////// right hand side ////////

		SetTextAlign(hDC, TA_LEFT);
		SetTextColor(hDC, 0x00FF6666);	// blue
		
		TextOut(hDC,(13.5*cw),(ch*3.7),"Inc.(Equ.)",10);
		TextOut(hDC,(13.5*cw),(ch*4.7),"Azimuth",7);
		TextOut(hDC,(13.5*cw),(ch*5.7),"Countdown",9);
		
		if(has_main_eng)
		{
		TextOut(hDC,(13.5*cw),(ch*8.7),"dV(Prograde)",12);
		TextOut(hDC,(13.5*cw),(ch*9.7),"Burn Duration",13);
		TextOut(hDC,(13.5*cw),(ch*10.7),"Burn Start",10);
		}
		else
		{
		TextOut(hDC,(13.5*cw),(ch*8.7),"Arc to Pe",9);
		TextOut(hDC,(13.5*cw),(ch*9.7),"T To Mid-Burn",13);
		}

		if(no_intersect)
		{
		if(warning_sw == 0)
		{
		SetTextColor(hDC, 0x005555FF);	// red
		warning_sw = 1;
		}
		else if(warning_sw == 1)
		{
		SetTextColor(hDC, 0x0033FFFF);	// red
		warning_sw = 0;
		}
		char kbuf[256];
		sprintf(kbuf,"INCLINATION OUT OF RANGE!");
		TextOut(hDC,(13.5*cw),(ch*6.7),kbuf,strlen(kbuf));
		}


		SetTextColor(hDC, 0x00FFFFFF);	// white
		{
		char kbuf[256];
		sprintf(kbuf,"LIFTOFF");
		TextOut(hDC,(13.5*cw),(ch*2.7),kbuf,strlen(kbuf));
		}

		{
		char kbuf[256];
		sprintf(kbuf,"TLI");
		TextOut(hDC,(13.5*cw),(ch*7.7),kbuf,strlen(kbuf));
		}
		
		//////////////////////////////////// data
		SetTextAlign(hDC, TA_RIGHT);

		{
		char kbuf[256];
		sprintf(kbuf,"%0.2f ", req_inc*DEG);
		TextOut(hDC,(37*cw),(ch*3.7),kbuf,strlen(kbuf));
		}

		{
		char kbuf[256];
		sprintf(kbuf,"%0.2f ", azimuth*DEG);
		TextOut(hDC,(37*cw),(ch*4.7),kbuf,strlen(kbuf));
		}

		{
		char kbuf[256];
		sprintf(kbuf,"T %0.1fs ", launch_time);
		TextOut(hDC,(36*cw),(ch*5.7),kbuf,strlen(kbuf));
		}

	if(has_main_eng)
	{
		{
		char kbuf[256];
		sprintf(kbuf,"%0.1fm/s", req_dV);
		TextOut(hDC,(37*cw),(ch*8.7),kbuf,strlen(kbuf));
		}

		{
		char kbuf[256];
		sprintf(kbuf,"%0.1fs  ", burn_duration);
		TextOut(hDC,(37*cw),(ch*9.7),kbuf,strlen(kbuf));
		}

		{
		char kbuf[256];
		sprintf(kbuf,"T %0.1fs  ", burn_time);
		TextOut(hDC,(37*cw),(ch*10.7),kbuf,strlen(kbuf));
		}
	}
	else
	{
		{
		char kbuf[256];
		sprintf(kbuf,"%0.2f", angle_to_pe*DEG);
		TextOut(hDC,(37*cw),(ch*8.7),kbuf,strlen(kbuf));
		}

		{
		char kbuf[256];
		sprintf(kbuf,"%0.1fs  ", time_to_pe);
		TextOut(hDC,(37*cw),(ch*9.7),kbuf,strlen(kbuf));
		}

	}


}


void LagrangeLaunchMFD::WriteStatus (FILEHANDLE scn) const
{
	//sprintf(oapiDebugString(), "WS");

	char vesname_scn[256];
	strcpy(vesname_scn, vesname);

	for (int i = 0; i < (int)strlen(vesname_scn); i++) 
	{
		if (vesname_scn[i] == ' ') 
		{
			vesname_scn[i] = '$';
		}
	}
	char bbuf[256];
	sprintf (bbuf, "%s", vesname_scn);
	oapiWriteScenario_string (scn, "LLVES", bbuf);

	char gbodyname_scn[256];
	strcpy(gbodyname_scn, gbodyname);

	for (int i = 0; i < (int)strlen(gbodyname_scn); i++) 
	{
		if (gbodyname_scn[i] == ' ') 
		{
			gbodyname_scn[i] = '$';
		}
	}
	char abuf[256];
	sprintf (abuf, "%s", gbodyname_scn);
	oapiWriteScenario_string (scn, "LLGBD", abuf);

	char dbuf[256];
	sprintf (dbuf, "%d %d %d %0.3f", status, lp_tgt, node_mode, parking_alt);
	oapiWriteScenario_string (scn, "LLPRM", dbuf);


}

void LagrangeLaunchMFD::ReadStatus (FILEHANDLE scn)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "LLVES", 5))
			sscanf_s (line+5, "%s", &vesname);
		else if (!strnicmp (line, "LLGBD", 5))
			sscanf_s (line+5, "%s", &gbodyname);
		else if (!strnicmp (line, "LLPRM", 5))
			sscanf_s (line+5, "%d%d%d%lf", &status, &lp_tgt, &node_mode, &parking_alt);
	}


	for (int i = 0; i < (int)strlen(vesname); i++) 
	{
		if (vesname[i] == '$') 
		{
			vesname[i] = ' ';
			//break;
		}
	}

	for (int i = 0; i < (int)strlen(gbodyname); i++) 
	{
		if (gbodyname[i] == '$') 
		{
			gbodyname[i] = ' ';
			//break;
		}
	}

	///////// init vessel /////////////////////
	if (strlen(vesname) == 0)
	{
	MyVes = oapiGetFocusObject();
	oapiGetObjectName(MyVes, vesname, 30);
	}
	else
	{
	MyVes = oapiGetVesselByName(vesname);
	}
	if(MyVes)vme = (VESSEL2*)oapiGetVesselInterface(MyVes);
	else
	{
	MyVes = oapiGetFocusObject();
	oapiGetObjectName(MyVes, vesname, 30);
	if(MyVes)vme = (VESSEL2*)oapiGetVesselInterface(MyVes);
	}


}


void LagrangeLaunchMFD::StoreStatus (void) const
{
	saveprm.sp1     = status;
	saveprm.sp2     = lp_tgt;
	saveprm.sp3		 = node_mode; 
	saveprm.sp4		 = parking_alt;
	/*saveprm.sp5		 = burn_sw;
	saveprm.sp6		= reentry_mode;
	saveprm.sp7		= aoa_alt_sw;
	saveprm.sp8		= aoa_timer;*/
	//sprintf(oapiDebugString(), "storestatus");
}

void LagrangeLaunchMFD::RecallStatus (void)
{
	
	status			= saveprm.sp1;
	lp_tgt			= saveprm.sp2;
	node_mode		= saveprm.sp3;
	parking_alt		= saveprm.sp4;
	/*auto_reentry_sw = saveprm.sp3;
	auto_burn_sw	= saveprm.sp4;
	burn_sw			= saveprm.sp5;
	reentry_mode	= saveprm.sp6;
	aoa_alt_sw		= saveprm.sp7;
	aoa_timer		= saveprm.sp8;*/
	//sprintf(oapiDebugString(), "recallstatus %0.1f", oapiGetSimTime());
}





